<?php
/**
 * @package n3t Fields
 * @author Pavel Poles - n3t.cz
 * @copyright (C) 2017-2019 - Pavel Poles - n3t.cz
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\Registry\Registry;

class n3tFieldsConvertor
{
  protected $installed = false;
  protected $names = array();

  public $overwrite = false;
  public $valuesCount = 0;
  public $overwriteCount = 0;

	public function __construct($data = array())
	{
	}

  public function installed()
  {
    return $this->installed;
  }

  public function fields()
  {
    return array();
  }

  public function contexts(&$languages)
  {
    $fields = $this->fields();

    $result = array();
    $languages = array();

    foreach ($fields as $field) {
      if (isset($this->names[$field->id]) && !empty($this->names[$field->id])) {
        $def = array(
          'name' => $this->names[$field->id],
          'label' => null,
          'description' => null,
          'type' => $field->type,
          'required' => $field->required,
          'size' => $field->size,
          'maxlength' => $field->maxlength,
          'default' => $field->default,
          'options' => $field->options,
        );

        if (!isset($result[$field->context]))
          $result[$field->context] = array();

        if (!isset($result[$field->context][$field->group_name])) {
          $group_def = array(
            'name' => $field->group_name,
            'label' => null,
            'description' => null,
            'fields' => array(),
          );

          if ($field->group_title) {
            $group_def['label'] = 'TPL_N3TFIELDS_FIELDSET_' . strtoupper($field->group_name) . '_LABEL';
            $languages[$group_def['label']] = $field->group_title;
          }

          if ($field->group_description) {
            $group_def['description'] = 'TPL_N3TFIELDS_FIELDSET_' . strtoupper($field->group_name) . '_DESC';
            $languages[$group_def['description']] = $field->group_description;
          }

          $result[$field->context][$field->group_name] = $group_def;
        }

        if ($field->title) {
          $def['label'] = 'TPL_N3TFIELDS_FIELD_' . strtoupper($this->names[$field->id]) . '_LABEL';
          $languages[$def['label']] = $field->title;
        }

        if ($field->description) {
          $def['description'] = 'TPL_N3TFIELDS_FIELD_' . strtoupper($this->names[$field->id]) . '_DESC';
          $languages[$def['description']] = $field->description;
        }

        $result[$field->context][$field->group_name]['fields'][] = $def;
      }
    }

    return $result;
  }

  public function convert()
  {
    return false;
  }

  protected function convertField($fieldId, $contextId, $value)
  {
    $fields = $this->fields();

    if ($value && isset($fields[$fieldId]) && isset($this->names[$fieldId])) {
      $field = $fields[$fieldId];
      $db = JFactory::getDbo();

      $query = $db->getQuery(true)
        ->select('*')
        ->from($db->quoteName('#__n3tfields'))
        ->where($db->quoteName('context') . ' = ' . $db->quote($field->context))
        ->where($db->quoteName('item_id') . ' = ' . $db->quote($contextId));
      $db->setQuery($query);
      $current = $db->loadObject();

      $values = new Registry();
      if ($current) {
        $values->loadString($current->value);
        if ($this->overwrite || !$values->get($this->names[$fieldId])) {
          $this->valuesCount++;
          if ($values->get($this->names[$fieldId]))
            $this->overwriteCount++;
          $values->set($this->names[$fieldId], $value);
          $current->value = $values->toString();
          $db->updateObject('#__n3tfields', $current, array('context', 'item_id'));
          return true;
        }
      } else {
        $this->valuesCount++;
        $current = new stdClass();
        $current->context = $field->context;
        $current->item_id = $contextId;
        $values->set($this->names[$fieldId], $value);
        $current->value = $values->toString();

        $db->insertObject('#__n3tfields', $current);
        return true;
      }
    }

    return false;
  }
}