<?php
/**
 * @package n3t Fields
 * @author Pavel Poles - n3t.cz
 * @copyright (C) 2017-2019 - Pavel Poles - n3t.cz
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

defined( '_JEXEC' ) or die( 'Restricted access' );

use Joomla\Registry\Registry;

JLoader::register('n3tFieldsConvertor', __DIR__ . '/../convertor.php');

class n3tFieldsConvertorFieldsAttach extends n3tFieldsConvertor
{

  public function __construct($data = array())
  {
    JLoader::register('fieldattach', 'components/com_fieldsattach/helpers/fieldattach.php');
    $this->installed = class_exists('fieldattach');

    if (isset($data['fieldsattach']))
      $this->names = $data['fieldsattach'];

    if (isset($data['fieldsattach_overwrite']))
      $this->overwrite = $data['fieldsattach_overwrite'];
  }

  public function fields()
  {
		if ($this->installed) {
      static $fields = null;

      if ($fields === null) {
        $db = JFactory::getDbo();

        $query = $db->getQuery(true)
          ->select($db->quoteName('f.id', 'id'))
          ->select($db->quoteName('f.title', 'title'))
          ->select($db->quoteName('f.type', 'type'))
          ->select($db->quoteName('f.required', 'required'))
          ->select($db->quoteName('f.extras', 'extras'))
          ->select($db->quoteName('g.id', 'group_id'))
          ->select($db->quoteName('g.title', 'group_title'))
          ->select($db->quoteName('g.group_for', 'group_for'))
          ->from($db->quoteName('#__fieldsattach', 'f'))
          ->join('LEFT', $db->quoteName('#__fieldsattach_groups', 'g') . ' ON (' . $db->quoteName('g.id') . ' = ' . $db->quoteName('f.groupid') . ')')
          ->where($db->quoteName('g.published') . ' = 1')
          ->where($db->quoteName('f.published') . ' = 1')
          ->order($db->quoteName(array('g.ordering', 'g.id', 'f.ordering', 'f.id')));
        $db->setQuery($query);
        $fields = $db->loadObjectList('id');

        foreach ($fields as &$field) {
          $field->context =  $field->group_for ? 'com_content.category' : 'com_content.article';
          $field->group_name = 'n3tfields' . $field->group_id;
          $field->group_description = null;
          $field->description = null;
          $field->size = null;
          $field->maxlength = null;
          $field->default = null;
          $field->options = null;

          switch($field->type) {
            case 'file':
              $field->type = 'text';
              break;
            case 'image':
              $field->type = 'media';
              break;
            case 'imagegallery':
              $field->type = 'n3tgallery';
              break;
            case 'input':
              $field->type = 'text';
              $options = preg_split('~\|~', $field->extras);
              if (count($options) > 0 && !empty($options[0]))
                $field->size = $options[0];
              if (count($options) > 1 && !empty($options[1]))
                $field->maxlength = $options[1];
              if (count($options) > 2 && !empty($options[2]))
                $field->default = $options[2];
              break;
            case 'select':
              $field->type = 'list';
              $field->options = array();
              $options = preg_split('~\r?\n~', $field->extras);
              foreach($options as $option) {
                $option = preg_split('~\|~', $option);
                if (count($option) > 1)
                  $field->options[$option[1]] = $option[0];
                if (count($option) > 2 && $option[2] == 'true')
                  $field->default = $option[1];
              }
              break;
            case 'textarea':
              if (strpos($field->extras, 'RichText') !== false)
                $field->type = 'editor';
              else
                $field->type = 'textarea';

              break;

            default:
              $field->type = 'text';
          }
        }
      }

      return $fields;
    }

    return array();
  }

  public function convert()
  {
    $db = JFactory::getDbo();
    $fields = $this->fields();

    $query = $db->getQuery(true)
      ->select($db->quoteName('f.articleid', 'item_id'))
      ->select($db->quoteName('f.fieldsid', 'fieldsid'))
      ->select($db->quoteName('f.value', 'value'))
      ->from($db->quoteName('#__fieldsattach_values', 'f'))
      ->order($db->quoteName(array('f.articleid', 'f.id')));
    $db->setQuery($query);
    $values = $db->loadObjectList();

    foreach ($values as $value)
      $this->convertField($value->fieldsid, $value->item_id, htmlspecialchars_decode($value->value));

    $query = $db->getQuery(true)
      ->select($db->quoteName('f.catid', 'item_id'))
      ->select($db->quoteName('f.fieldsid', 'fieldsid'))
      ->select($db->quoteName('f.value', 'value'))
      ->from($db->quoteName('#__fieldsattach_categories_values', 'f'))
      ->order($db->quoteName(array('f.catid', 'f.id')));
    $db->setQuery($query);
    $values = $db->loadObjectList();

    foreach ($values as $value)
      $this->convertField($value->fieldsid, $value->item_id, htmlspecialchars_decode($value->value));


    foreach ($fields as $field) {
      if ($field->type == 'n3tgallery'
      && isset($this->names[$field->id])) {
        $isCategory = $field->context == 'com_content.category';
        $query = $db->getQuery(true);
        if ($isCategory)
          $query
            ->select($db->quoteName('f.catid', 'item_id'))
            ->order($db->quoteName(array('f.catid', 'f.ordering', 'f.id')));
        else
          $query
            ->select($db->quoteName('f.articleid', 'item_id'))
            ->order($db->quoteName(array('f.articleid', 'f.ordering', 'f.id')));
        $query
          ->select($db->quoteName('f.title', 'title'))
          ->select($db->quoteName('f.image1', 'image'))
          ->from($db->quoteName('#__fieldsattach_images', 'f'))
          ->where($db->quoteName('f.fieldsattachid') . '=' . $field->id);
        $db->setQuery($query);
        $values = $db->loadObjectList();


        if ($values) {
          $limiter = new stdClass();
          $limiter->item_id = -1;
          $limiter->image = null;
          $limiter->title = null;
          $values[] = $limiter;

          $itemId = $values[0]->item_id;

          $images = array();
          foreach ($values as $value) {
            if ($itemId != $value->item_id) {
              $this->convertField($field->id, $itemId, json_encode($images));

              $itemId = $value->item_id;
              $images = array();
            }

            if ($value->image)
              $images[] = array('image' => $value->image, 'title' => $value->title);
          }
        }
      }
    }
    return true;
  }

}