<?php
/**
 * @package n3t Fields
 * @author Pavel Poles - n3t.cz
 * @copyright (C) 2017-2019 - Pavel Poles - n3t.cz
 * @license GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
**/

defined('_JEXEC') or die;

use Joomla\CMS\Access\Exception\NotAllowed;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\Factory;
use Joomla\CMS\Filesystem\File;
use Joomla\CMS\Form\Form;
use Joomla\CMS\Form\FormHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Plugin\CMSPlugin;
use Joomla\Registry\Registry;

class plgSystemN3tFields extends CMSPlugin {

  protected $autoloadLanguage = true;
  protected $app = null;

  public function __construct(&$subject, $config = [])
  {
    parent::__construct($subject, $config);

    $this->app = Factory::getApplication();
  }

  protected function getTemplates()
  {
    static $templates = null;

    if ($templates === null) {
      $db = Factory::getDbo();
      $query = $db->getQuery(true)
        ->select($db->quoteName('a.name'))
        ->from($db->quoteName('#__extensions', 'a'))
        ->where($db->quoteName('a.client_id') . ' = 0')
        ->where($db->quoteName('a.enabled') . ' = 1')
        ->where($db->quoteName('a.type') . ' = ' . $db->quote('template'))
        ->order($db->quoteName('a.name') . ' ASC');
      $db->setQuery($query);
      $templates = $db->loadColumn();
    }

    return $templates;
  }

  protected function getFiles($name)
  {
    $files = array();
    $file = str_replace('.', '/', $name) . '.xml';

    foreach ($this->getTemplates() as $template) {
      if (File::exists(JPATH_SITE . '/templates/' . $template . '/n3tfields/' . $file))
        $files[] = [
          'path' => JPATH_SITE . '/templates/' . $template . '/n3tfields/' . $file,
          'template' => $template,
        ];
    }

    return $files;
  }

  protected function getContext($context)
  {
    // com_categories
    if (strpos($context, 'com_categories.category') === 0)
      $context = $this->app->input->getCmd('extension') . '.category';
    // Users profile aliases
    else if (in_array($context, array('com_users.profile', 'com_users.user', 'com_admin.profile')))
      $context = 'com_users.user';
    // Frontend module manager
    else if ($this->app->isClient('site') && $context == 'com_config.module')
      $context = 'com_modules.module';
    // Regular Labs Advanced module manager
    else if ($context == 'com_advancedmodules.module')
      $context = 'com_modules.module';

    return $context;
  }

  public function onAfterInitialise()
  {
    JLoader::register('n3tfields', __DIR__ . '/helpers/n3tfields.php');
    n3tFields::registerContext('article', 'com_content.article');
    n3tFields::registerContext('banner', 'com_banners.banner');
    n3tFields::registerContext('category', 'com_content.category');
    n3tFields::registerContext('contact', 'com_contacts.contact');
    n3tFields::registerContext('menu', 'com_menus.item');
    n3tFields::registerContext('module', 'com_modules.module');
    n3tFields::registerContext('tag', 'com_tags.tag');
    n3tFields::registerContext('user', 'com_users.user');
  }

  public function onContentPrepareForm($form, $data)
  {
    if (!($form instanceof Form)) {
      $this->_subject->setError('JERROR_NOT_A_FORM');
      return false;
    }

    $data = (array)$data;
    $context = $this->getContext($form->getName());

    $files = $this->getFiles($context);

    // Context extensions
    switch ($context) {
      case 'com_modules.module':
        if (isset($data['module']))
          $files = array_merge($files, $this->getFiles($context . '.' . $data['module']));
        break;
      case 'com_menus.item':
        if (isset($data['component_id'])) {
          $components = ComponentHelper::getComponents();
          foreach($components as $component) {
            if ($component->id == $data['component_id']) {
              $files = array_merge($files, $this->getFiles($context . '.' . $component->option));
              break;
            }
          }
        }
        break;
    }

    $lang = Factory::getLanguage();

    JLoader::register('FieldsHelper', JPATH_ADMINISTRATOR . '/components/com_fields/helpers/fields.php');
    $fieldTypes = FieldsHelper::getFieldTypes();
    foreach ($fieldTypes as $field) {
      if ($path = $field['path'])
        FormHelper::addFieldPath($path);

      if ($path = $field['rules'])
        FormHelper::addRulePath($path);
    }

    foreach ($files as $file) {
      $lang->load('tpl_' . $file['template'] , JPATH_SITE . '/templates/' . $file['template'], $lang->getTag(), true);
      $form->loadFile($file['path'], false);
    }

    return true;
  }

  public function onContentPrepareData($context, $data)
  {
    if (is_array($data) && isset($data['id']))
      $id = $data['id'];
    elseif (is_object($data) && isset($data->id))
      $id = $data->id;
    else
      return true;

    if (empty($id))
      return true;

    $context = $this->getContext($context);

    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select($db->quoteName('value'))
      ->from('#__n3tfields')
      ->where($db->quoteName('context') . ' = ' . $db->quote($context))
      ->where($db->quoteName('item_id') . ' = ' . $id);
    $db->setQuery($query);

    if (is_array($data)) {
      $data['n3tfields'] = new Registry();
      $data['n3tfields'] = $data['n3tfields']->loadString($db->loadResult());
    } else {
      $data->n3tfields = new Registry();
      $data->n3tfields = $data->n3tfields->loadString($db->loadResult());
    }

    return true;
  }

  public function onContentAfterSave($context, $item, $isNew, $data = array())
  {
    if (empty($data))
      $data = $this->app->input->post->get('jform', array(), 'array');

    if (!is_array($data) || empty($item->id))
      return true;

    if (!isset($data['n3tfields']) || !is_array($data['n3tfields']))
      return true;

    $context = $this->getContext($context);

    $value = new stdClass();
    $value->context = $context;
    $value->item_id = $item->id;
    $value->value = new Registry();
    $value->value->loadArray($data['n3tfields']);
    $value->value = $value->value->toString();

    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->select($db->quoteName('item_id'))
      ->from('#__n3tfields')
      ->where($db->quoteName('context') . ' = ' . $db->quote($context))
      ->where($db->quoteName('item_id') . ' = ' . $item->id);
    $db->setQuery($query);

    if ($db->loadResult())
      $db->updateObject('#__n3tfields', $value, array('context', 'item_id'));
    else
      $db->insertObject('#__n3tfields', $value);

    n3tFields::cleanCache($context, $item);

    return true;
  }

  public function onContentAfterDelete($context, $item)
  {
    if (empty($item->id))
      return true;

    $context = $this->getContext($context);

    $db = Factory::getDbo();
    $query = $db->getQuery(true)
      ->delete('#__n3tfields')
      ->where($db->quoteName('context') . ' = ' . $db->quote($context))
      ->where($db->quoteName('item_id') . ' = ' . $item->id);
    $db->setQuery($query);
    $db->execute();

    n3tFields::cleanCache($context, $item);

    return true;
  }

  public function onExtensionAfterSave($context, $item, $isNew)
  {
    return $this->onContentAfterSave($context, $item, $isNew, $data);
  }

  public function onExtensionAfterDelete ($context, $item)
  {
    return $this->onContentAfterDelete($context, $item);
  }

  public function onUserAfterSave($user, $isNew, $success, $msg)
  {
    if (!$user['id'] || !$success)
      return true;

    return $this->onContentAfterSave('com_users.user', Factory::getUser($user['id']), $isNew, $data);
  }

  public function onUserAfterDelete($user, $succes, $msg)
  {
    return $this->onContentAfterDelete('com_users.user', (object)$item);
  }

  public function onAjaxn3tFields()
  {
    if (!Factory::getUser()->authorise('com_plugins.edit'))
      throw new Notallowed(Text::_('JERROR_ALERTNOAUTHOR'), 403);

    $return = array(
      'contexts' => array(),
    );

    $input = $this->app->input;
    $data = $input->post->get('jform', array(), 'array');

    if ($input->getCmd('convertor')) {
      $class = 'n3tFieldsConvertor' . $input->getCmd('convertor');
      JLoader::register($class, __DIR__ . '/helpers/convertor/' . $input->getCmd('convertor') . '.php');
      if (class_exists($class)) {
        $convertor = new $class(@$data['params']);

        if ($convertor->installed()) {
          if ($input->getCmd('converttask') == 'convert') {
            $this->app->enqueueMessage(Text::_('PLG_SYSTEM_N3TFIELDS_CONVERTING'), 'info');
            if ($convertor->overwrite)
              $this->app->enqueueMessage(Text::_('PLG_SYSTEM_N3TFIELDS_CONVERTING_OVERWRITE'), 'info');

            $return['convert'] = $convertor->convert();
          }

          $contexts = $convertor->contexts($languages);

          if ($input->getCmd('converttask') == 'convert') {
            $fieldsCount = 0;
            $groupsCount = 0;
            foreach($contexts as $context => $groups) {
              if (empty($groups))
                unset($contexts[$context]);
              else {
                $groupsCount += count($groups);
                foreach ($groups as $group)
                  $fieldsCount += count($group);
              }
            }

            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED'), 'success');
            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED_CONTEXTS', count($contexts)), 'success');
            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED_GROUPS', $groupsCount), 'success');
            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED_FIELDS', $fieldsCount), 'success');
            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED_VALUES', $convertor->valuesCount), 'success');
            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED_OVERWRITES', $convertor->overwriteCount), 'success');
            $this->app->enqueueMessage(Text::sprintf('PLG_SYSTEM_N3TFIELDS_CONVERTED_LANGUAGES', count($languages)), 'success');
          }

          $this->app->enqueueMessage(Text::_('PLG_SYSTEM_N3TFIELDS_CRETAING_CONTEXTS'), 'info');
          JLoader::register('n3tFieldsXml', __DIR__ . '/helpers/xml.php');
          foreach($contexts as $context => $fields)
            $return['contexts'][$context] = n3tFieldsXml::fieldsToXml($fields);

          $return['languages'] = implode("\n", array_map(
            function ($key, $value) {
              return $key . '="' . $value . '"';
            },
            array_keys($languages), $languages
          ));
          $this->app->enqueueMessage(Text::_('PLG_SYSTEM_N3TFIELDS_CREATED_CONTEXTS'), 'success');
        }
      }
    }

    return $return;
  }

}
